@@ -1,7 +1,4 @@
 package Dist::Dzpl::Parser;
-BEGIN {
-  $Dist::Dzpl::Parser::VERSION = '0.0010';
-}
 
 use strict;
 use warnings;
@@ -126,28 +123,3 @@ sub _parse_license {
 
 1;
 
-
-__END__
-=pod
-
-=head1 NAME
-
-Dist::Dzpl::Parser
-
-=head1 VERSION
-
-version 0.0010
-
-=head1 AUTHOR
-
-  Robert Krimen <robertkrimen@gmail.com>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2010 by Robert Krimen.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=cut
-
